
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.microedition.midlet.*;
import javax.microedition.rms.InvalidRecordIDException;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreFullException;
import javax.microedition.rms.RecordStoreNotFoundException;
import javax.microedition.rms.RecordStoreNotOpenException;
import javax.microedition.lcdui.*;

import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

/**
 * 
 * Entre 2 Almas	[https://sourceforge.net/projects/entre2almas/]  
 * ------------------------------------------------------------------------
 * This file is part of "Entre 2 Almas".
 * "Entre 2 Almas" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * "Entre 2 Almas" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "Entre 2 Almas"; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ------------------------------------------------------------------------
 * 		Copyright 2005 - 2006 Pedro Javier Sez Martnez
 * 
 * Clase principal del juego Entre2Almas, extiende de MIDlet y por ello esta
 * clase ser la que primero se ejecute al iniciar el midlet
 * 
 * @author Pedro Javier Sez Martnez
 * @version 1.0
 */
public class Entre2Almas extends MIDlet {

	/**
	 * Representa el manejador de la pantalla, y solo habr una instancia de
	 * Display por midlet
	 */
	private Display display;

	/**
	 * Indica si se debe mostrar el Splash Screen
	 */
	private boolean isSplash = true;

	/**
	 * Indica si tenemos un men clsico (0) o avanzado (1);
	 */
	public int tipoMenu;

	/**
	 * La interfaz de usuario del men, de tipo estndar con fondo de pantalla.
	 * (Avanzado)
	 */
	private UITerminalSTBG avanzado;

	/**
	 * La interfaz de usuario del men, de tipo estndar. (Clsico)
	 */
	private UITerminalST clasico;

	/**
	 * Ser la base de datos del juego.
	 */
	private RecordStore rs;

	/**
	 * Ser la base de datos temporal del juego. Se utilizar para a la hora de
	 * guardar la partida por si acaso ocurriese que el mvil se queda sin
	 * batera, o que se apaga de repente, o algo raro, el guardar partida se
	 * hace primero en un rms temporal, y luego, cuando se ha completado, se
	 * pasa al bueno.
	 */
	private RecordStore rsTemp;

	/**
	 * Si es 0 indica que el men activo es clsico y viceversa
	 */
	public int menuActivo;

	/**
	 * Donde se desarrollar el juego
	 */
	public Ejecucion gameCanvas;

	/**
	 * Parser utilizado para cargar los datos bsicos del juego. Como son el
	 * splash screen y las imgenes del men.
	 */
	private KXmlParser parser;

	/**
	 * Guarda aquellos datos necesarios para restaurar el juego por donde estaba
	 * en un futura partida.
	 */
	public Vector preguardado;

	/**
	 * Mtodo llamado cuando se inicia la aplicacin.(heredado de MIDlet)
	 */
	public void startApp() {
		try {
			preguardado = new Vector();
			//Recupera una instancia del Display, que ser la nica
			//que se usar durante toda la ejecucin.
			display = Display.getDisplay(this);
			//Inicializamos el parser
			parser = new KXmlParser();
			parser.setInput(new InputStreamReader(this.getClass()
					.getResourceAsStream("/basicos.xml")));
			cargarDatos();
			//Se inicializa primero ui, porque luego es utilizado en el splash
			//adems como es el primer canvas que se inicializa, pues ser la
			//nica clase que llamar al mtodo Utils.grupo();
			avanzado = new UITerminalSTBG(this);
			clasico = new UITerminalST(this);

			tipoMenu = getMenuRs();
			menuActivo = tipoMenu;
			//Define a que grupo, en cuanto a tamao de las imgenes,
			//corresponde este mvil
			cargarSplash();
		} catch (XmlPullParserException e) {
			mostrarAlerta("Entre2Almas::startApp::Problemas con el XML: " + e,
					AlertType.ERROR);
			e.printStackTrace();
		} catch (IOException e1) {
			mostrarAlerta("Entre2Almas::startApp::Error de entrada y salida: "
					+ e1, AlertType.ERROR);
			e1.printStackTrace();
		}
	}

	/**
	 * devuelve la referencia al display
	 * 
	 * @return Referencia al display de la aplicacin.
	 */
	public Display getDisplay() {
		return display;
	}

	/**
	 * Mtodo llamado cuando se pausa el juego (heredado de MIDlet)
	 */
	public void pauseApp() {
	}

	/**
	 * Mtodo llamado cuando se destruye el juego (heredado de MIDlet)
	 */
	public void destroyApp(boolean unconditional) {
		System.gc();
		notifyDestroyed();
	}

	/**
	 * Carga la pantalla de presentacin
	 */
	private void cargarSplash() {
		if (isSplash) {
			isSplash = false;
			//Llamamos a SplashScreen
			if (tipoMenu == 0)
				new SplashScreen(this, clasico, 2000);
			else
				new SplashScreen(this, avanzado, 2000);
		}
	}

	/**
	 * A partir de lo que el usuario elija en las opciones, cargar el men
	 * clsico o el avanzado
	 */
	public void menu() {
		//Entonces se supone que vamos a cargar un men de distinto
		//formato y por tanto deberemos empezar por la primera opcin
		Utils.menuPrinIdx = 0;
		if (tipoMenu == 0) {
			if (Utils.jugando)
				Utils.menuPrinIdx = 0;
			Utils.cargada = -1;
			Utils.menuAyudaIdx = 0;
			menuActivo = 0;
			Utils.parteMenu = Utils.menuPrin;
			display.setCurrent(clasico);
		} else {
			//Ajustamos las variables que han podido ser cambiadas
			//anteriormente (por ejemplo mientras jugamos)
			if (Utils.jugando) {
				avanzado.ajustarVariables();
				Utils.menuPrinIdx = 0;
			}
			menuActivo = 1;
			//Decimos que la prxima vez que entremos al men, que nos
			//recargue el men (as ya saldr lo de volver a partida)
			//y adems le decimos que la parte de men en la que estamos es
			//el men principal
			Utils.cargada = -1;
			Utils.menuAyudaIdx = 0;
			Utils.parteMenu = Utils.menuPrin;
			display.setCurrent(avanzado);
		}

	}

	/**
	 * Devuelve el tipo de men que en ese momento esta seleccionado en las
	 * opciones
	 * 
	 * @return El tipo de men que en ese momento esta seleccionado en las
	 *         opciones
	 */
	public String getTipoMenu() {
		if (tipoMenu == 0)
			return "Clsico";
		else
			return "Avanzado";
	}

	/**
	 * Iniciamos la partida.
	 */
	public void IniciarPartida() {
		//Si ya estbamos jugando otra partida anteriormente
		if (Utils.jugando) {
			gameCanvas.salir = true;
			synchronized (gameCanvas) {
				gameCanvas.notify();
			}
		}
		Utils.jugando = true;
		//recogemos las referencias de las clases
		//el gameCanvas o la partida no se crea hasta que elegimos
		// las opciones, por eso no lo creamos en el
		//startApp como al resto.
		gameCanvas = null;
		System.gc();
		gameCanvas = new Ejecucion(this);
		gameCanvas.start();
		display.setCurrent(gameCanvas);
		//y cuando la hemos puesto en marcha, la mostramos por pantalla.
	}

	/**
	 * Reanuda la partida, que previamente se haba parado para salir al men
	 * principal
	 */
	public void seguirJugando() {
		//Reajustamos las variables, que han sido modificadas,
		//del GameCanvas
		gameCanvas.ajustarVariables();
		//Despertamos la Ejecucin del GameCanvas
		synchronized (gameCanvas) {
			gameCanvas.notify();
		}
		//y cuando la hemos puesto en marcha, la mostramos por pantalla.
		display.setCurrent(gameCanvas);
		System.gc();
	}

	/**
	 * salimos de la aplicacin.
	 */
	public void salir() {
		destroyApp(true);
	}

	/**
	 * A partir de un chorro de bytes que haba en un registro, nos devuelve el
	 * ultimo men que utilizamos en la anterior partida.
	 * 
	 * @return el tipo de men
	 */
	private int getMenuRs() {
		try {
			rs = RecordStore.openRecordStore("menu", true);
			//			getNumRecords devuelve el numero de registros
			//almacenados en un determinado recordStore.
			//Si no hay ninguno significa que no hay nada guardado en la BBDD
			if (rs.getNumRecords() == 0) {
				byte[] registro;
				//usamos Streams para leer registros en el rs, as
				//almacenamos informacin de distinto tipo en el
				//mismo registro, y adems al leer podemos distinguir
				//entre la info de un tipo u otro.
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				DataOutputStream dos = new DataOutputStream(baos);
				dos.writeInt(1);//men avanzado
				dos.flush();
				registro = baos.toByteArray();
				//aade un registro en el recordStore, en este caso aade
				//lo que hay en el array registro desde el byte 0 hasta
				//su ultimo byte.
				rs.addRecord(registro, 0, registro.length);
				baos.close();
				dos.close();
				registro = null;
			}
			//			En el recordStore los parametros no van de 0 a N
			//sino de 1 a N
			byte[] registro = rs.getRecord(1);
			//Explicado el por que del uso de streams en
			//initRecord()
			ByteArrayInputStream bais = new ByteArrayInputStream(registro);
			DataInputStream dis = new DataInputStream(bais);
			int tipoMenu = dis.readInt();
			bais.close();
			dis.close();
			rs.closeRecordStore();
			return tipoMenu;
		} catch (Exception e) {
			try {
				rs.closeRecordStore();
			} catch (RecordStoreNotFoundException e1) {
				mostrarAlerta("Entre2Almas::getMenuRs::getMenuRs::Error RMS: "
						+ e, AlertType.ERROR);
				e1.printStackTrace();
			} catch (RecordStoreException e1) {
				mostrarAlerta("Entre2Almas::getMenuRs::getMenuRs::Error RMS: "
						+ e, AlertType.ERROR);
				e1.printStackTrace();
			}
			mostrarAlerta("Entre2Almas::getMenuRs::Error desconocido: " + e,
					AlertType.ERROR);
			return -1;
		}
	}

	/**
	 * Actualiza la base de datos de men
	 */
	public void setMenuRs() {
		try {
			//Utilizamos streams debido a la utilidad que tienen
			//explicada en el mtodo verRecords()
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			DataOutputStream dos = new DataOutputStream(baos);
			dos.writeInt(tipoMenu);
			dos.flush();
			//recoge todo lo almacenado a travs de dos en el array
			//de bytes nreg.
			byte[] registro = baos.toByteArray();
			baos.close();
			dos.close();

			rs = RecordStore.openRecordStore("menu", true);
			rs.setRecord(1, registro, 0, registro.length);

			rs.closeRecordStore();
		} catch (Exception e) {
			System.out.println("RMS:setMenuRs::" + e);
			try {
				rs.closeRecordStore();
			} catch (RecordStoreNotFoundException e1) {
				mostrarAlerta("Entre2Almas::setMenuRs::Error RMS: " + e,
						AlertType.ERROR);
				e1.printStackTrace();
			} catch (RecordStoreException e1) {
				mostrarAlerta("Entre2Almas::setMenuRs::Error RMS: " + e,
						AlertType.ERROR);
				e1.printStackTrace();
			}
			mostrarAlerta("Entre2Almas::setMenuRs::Error desconocido: " + e,
					AlertType.ERROR);
		}
	}

	/**
	 * Carga los datos necesarios, del archivo XML, para mostrar el men y el
	 * splash screen
	 * 
	 * @throws XmlPullParserException
	 *             Excepcin del parser
	 * @throws IOException
	 *             Excepcin de entrada/salida
	 */
	private void cargarDatos() throws XmlPullParserException, IOException {
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "Basicos");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "Imagenes");
		parser.nextTag();

		//Numero total de imagenes que tendr el array de imagenes
		parser.require(XmlPullParser.START_TAG, null, "numImagenes");
		int numImagenes = Integer.parseInt(parser.nextText());
		Utils.imagenes = new String[numImagenes];

		parser.require(XmlPullParser.END_TAG, null, "numImagenes");
		StringBuffer temp = new StringBuffer();
		StringBuffer imagen = new StringBuffer();
		int i = 0;

		//Cargamos el array de imgenes
		while (parser.nextTag() != XmlPullParser.END_TAG) {
			if (parser.getName().equals("Imagen")) {
				parser.require(XmlPullParser.START_TAG, null, "Imagen");
				temp.append(parser.getAttributeValue(0) + ";");
				parser.nextTag();
				parser.require(XmlPullParser.START_TAG, null, "id");
				imagen.append(parser.nextText() + ";" + temp.toString());
				parser.require(XmlPullParser.END_TAG, null, "id");
				parser.nextTag();
				parser.require(XmlPullParser.START_TAG, null, "tamX");
				imagen.append(parser.nextText() + ";");
				parser.require(XmlPullParser.END_TAG, null, "tamX");
				parser.nextTag();
				parser.require(XmlPullParser.START_TAG, null, "tamY");
				imagen.append(parser.nextText());
				parser.require(XmlPullParser.END_TAG, null, "tamY");
				parser.nextTag();
				parser.require(XmlPullParser.END_TAG, null, "Imagen");
				Utils.imagenes[i++] = imagen.toString();
				temp.delete(0, temp.length());
				imagen.delete(0, imagen.length());
			}
			System.gc();
		}
		temp = null;
		imagen = null;
		parser.require(XmlPullParser.END_TAG, null, "Imagenes");
		parser.nextTag();

		//Cargamos el sprite del splash screen
		parser.require(XmlPullParser.START_TAG, null, "splash");
		Utils.splash = getDatosImagen().toString();
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "splash");

		//Cargamos la imagenes del men.
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "menu");
		parser.nextTag();

		//Primero la del men principal
		parser.require(XmlPullParser.START_TAG, null, "menuPrin");
		Utils.titulo = parser.getAttributeValue(0);
		Utils.imMenuPrin = getDatosImagen().toString();
		//cargamos las opciones del men principal
		cargaOpMenuPrin();
		parser.require(XmlPullParser.END_TAG, null, "menuPrin");
		parser.nextTag();

		//Luego la imagen de opciones
		parser.require(XmlPullParser.START_TAG, null, "menuOpciones");
		Utils.imOpciones = getDatosImagen().toString();
		parser.nextTag();
		//y su opcin tipo de men.
		parser.require(XmlPullParser.START_TAG, null, "tipoMenu");
		Utils.tiposMenuTxt = parser.getAttributeValue(0);
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "clasico");
		Utils.tiposMenu[0] = parser.nextText();
		parser.require(XmlPullParser.END_TAG, null, "clasico");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "avanzado");
		Utils.tiposMenu[1] = parser.nextText();
		parser.require(XmlPullParser.END_TAG, null, "avanzado");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "tipoMenu");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "menuOpciones");
		parser.nextTag();
		
		System.gc();
		
		//Luego la imagen de memoria
		parser.require(XmlPullParser.START_TAG, null, "memoria");
		Utils.imMemoria = getDatosImagen().toString();
		//		cargamos las opciones del men principal
		cargaOpMemo();
		parser.require(XmlPullParser.END_TAG, null, "memoria");
		parser.nextTag();

		//Imagen de ayuda
		parser.require(XmlPullParser.START_TAG, null, "ayuda");
		Utils.imAyuda = getDatosImagen().toString();
		//		cargamos las opciones y el contenido de sus sub-ayudas
		cargaOpAyuda();
		parser.require(XmlPullParser.END_TAG, null, "ayuda");
		parser.nextTag();

		//Imagen de acerca de...
		parser.require(XmlPullParser.START_TAG, null, "acerca");
		Utils.imAcerca = getDatosImagen().toString();
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "contenido");
		Utils.conteAcerca = parser.nextText();
		parser.require(XmlPullParser.END_TAG, null, "contenido");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "acerca");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "menu");
		parser.nextTag();

		//Sprite de cargando
		parser.require(XmlPullParser.START_TAG, null, "cargando");
		Utils.spCargando = getDatosImagen().toString();
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "cargando");
		parser.nextTag();

		//		Sprite de guardando
		parser.require(XmlPullParser.START_TAG, null, "guardando");
		Utils.imGuardando = getDatosImagen().toString();
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "guardando");
		parser.nextTag();

		//Sprite de puntero
		parser.require(XmlPullParser.START_TAG, null, "puntero");
		Utils.spPuntero = getDatosImagen().toString();
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "puntero");
		parser.nextTag();

		//Imagen del puntero que comprueba el area caminable
		parser.require(XmlPullParser.START_TAG, null, "punCami");
		Utils.imPunCami = getDatosImagen().toString();
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "punCami");

		//Imagen temporal para cargar el inventario
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "imTmpInv");
		Utils.imTmpInv = getDatosImagen().toString();
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "imTmpInv");
		
		System.gc();
		
		//Cargamos los datos del protagonista.
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "apariencias");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "numApariencias");
		//		Cargamos el array de apariencias
		Utils.apariencias = new String[Integer.parseInt(parser.nextText()) * 4];
		parser.require(XmlPullParser.END_TAG, null, "numApariencias");
		while (parser.nextTag() != XmlPullParser.END_TAG) {
			cargarApariencia();
		}
		parser.require(XmlPullParser.END_TAG, null, "apariencias");
		//una vez cargadas, activamos una cualquiera
		Utils.actualizarApariencia(0);

		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "Basicos");
		parser = null;
		
		System.gc();
	}

	/**
	 * Recupera, a partir de un archivo XML, los datos descriptivos de una
	 * imagen
	 * 
	 * @throws IOException
	 *             Excepcin de entrada/salida
	 * @throws XmlPullParser
	 *             Exception Excepcin en el XML
	 */
	public StringBuffer getDatosImagen() throws XmlPullParserException,
			IOException {
		StringBuffer imagen = new StringBuffer();
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "id");
		imagen.append(parser.nextText() + ";");
		parser.require(XmlPullParser.END_TAG, null, "id");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "frameIni");
		imagen.append(parser.nextText() + ";");
		parser.require(XmlPullParser.END_TAG, null, "frameIni");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "numFrames");
		imagen.append(parser.nextText());
		parser.require(XmlPullParser.END_TAG, null, "numFrames");
		System.gc();
		return imagen;
	}

	/**
	 * Carga del archivo basicos.xml, las nombre de las opciones del Men
	 * Principal.
	 * 
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private void cargaOpMenuPrin() throws XmlPullParserException, IOException {
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "opciones");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "opSegJug");
		Utils.opMenuPrin[0] = parser.getAttributeValue(0);
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "opSegJug");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "opNuevaPartida");
		Utils.opMenuPrin[1] = parser.getAttributeValue(0);
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "opNuevaPartida");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "opMemoria");
		Utils.opMenuPrin[2] = parser.getAttributeValue(0);
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "opMemoria");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "opOpciones");
		Utils.opMenuPrin[3] = parser.getAttributeValue(0);
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "opOpciones");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "opAyuda");
		Utils.opMenuPrin[4] = parser.getAttributeValue(0);
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "opAyuda");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "opAcerca");
		Utils.opMenuPrin[5] = parser.getAttributeValue(0);
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "opAcerca");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "opciones");
		parser.nextTag();
	}

	/**
	 * Carga del archivo basicos.xml, las nombre de las opciones del Men
	 * Memoria.
	 * 
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private void cargaOpMemo() throws XmlPullParserException, IOException {
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "opciones");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "guardar");
		Utils.opMenuMemo[0] = parser.getAttributeValue(0);
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "guardar");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "cargar");
		Utils.opMenuMemo[1] = parser.getAttributeValue(0);
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "cargar");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "opciones");
		parser.nextTag();
	}

	/**
	 * Carga del archivo basicos.xml, las nombre de las opciones del Men Ayuda.
	 * Adems tambin cargar el contenido de cada una de esta sub-ayudas.
	 * 
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private void cargaOpAyuda() throws XmlPullParserException, IOException {
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "opciones");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "tematica");
		Utils.opMenuAyuda[0] = parser.getAttributeValue(0);
		Utils.txtSubAyuda[0] = parser.nextText();
		parser.require(XmlPullParser.END_TAG, null, "tematica");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "teclasGenerales");
		Utils.opMenuAyuda[1] = parser.getAttributeValue(0);
		Utils.txtSubAyuda[1] = parser.nextText();
		parser.require(XmlPullParser.END_TAG, null, "teclasGenerales");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "enVideo");
		Utils.opMenuAyuda[2] = parser.getAttributeValue(0);
		Utils.txtSubAyuda[2] = parser.nextText();
		parser.require(XmlPullParser.END_TAG, null, "enVideo");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "enInventario");
		Utils.opMenuAyuda[3] = parser.getAttributeValue(0);
		Utils.txtSubAyuda[3] = parser.nextText();
		parser.require(XmlPullParser.END_TAG, null, "enInventario");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "enMapa");
		Utils.opMenuAyuda[4] = parser.getAttributeValue(0);
		Utils.txtSubAyuda[4] = parser.nextText();
		parser.require(XmlPullParser.END_TAG, null, "enMapa");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "conversando");
		Utils.opMenuAyuda[5] = parser.getAttributeValue(0);
		Utils.txtSubAyuda[5] = parser.nextText();
		parser.require(XmlPullParser.END_TAG, null, "conversando");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "opciones");
		parser.nextTag();
	}

	/**
	 * Carga una apariencia de nuestro personaje principal
	 * 
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	public void cargarApariencia() throws XmlPullParserException, IOException {
		parser.require(XmlPullParser.START_TAG, null, "apariencia");
		int id = Integer.parseInt(parser.getAttributeValue(0));
		id = id * 4;
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "lado");
		Utils.apariencias[id] = getDatosImagen().toString();
		id++;
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "lado");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "espalda");
		Utils.apariencias[id] = getDatosImagen().toString();
		id++;
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "espalda");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "frente");
		Utils.apariencias[id] = getDatosImagen().toString();
		id++;
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "frente");
		parser.nextTag();
		parser.require(XmlPullParser.START_TAG, null, "cara");
		Utils.apariencias[id] = getDatosImagen().toString();
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "cara");
		parser.nextTag();
		parser.require(XmlPullParser.END_TAG, null, "apariencia");
	}

	/**
	 * Guardar en preguardado los resultados necesarios para restablecer el
	 * juego al momento de la ltima partida guardada.
	 * 
	 * @param res
	 *            Resultado a almacenar.
	 */
	public void pregarduar(Resultado res) {
		//no se pueden meter entero en un objeto Vector
		preguardado.addElement(res);
	}

	/**
	 * Guarda, los datos necesarios de la partida actual, en la base de datos.
	 *  
	 */
	public void guardarDatos() {
		try {
			//Por si acaso ocurriese que el mvil se queda sin batera, o que
			// se
			//apaga de repente, o algo raro, el guardar partida se hace primero
			// en
			//un rms temporal, y luego, cuando se ha completado, se pasa al
			// bueno.
			rsTemp = RecordStore.openRecordStore("temporal", true);

			//			usamos Streams para leer registros en el rs, as
			//almacenamos informacin de distinto tipo en el
			//mismo registro, y adems al leer podemos distinguir
			//entre la info de un tipo u otro.
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			DataOutputStream dos = new DataOutputStream(baos);

			//		Fase actual
			dos.writeInt(gameCanvas.nFase);
			//		Mapa actual
			dos.writeInt(gameCanvas.mapaActual);
			//		Estancia actual
			dos.writeInt(((Mapa) (gameCanvas.mapas
					.search(gameCanvas.mapaActual).data)).activa);
			//Escenario actual
			dos.writeInt(((Mapa) (gameCanvas.mapas
							.search(gameCanvas.mapaActual).data))
							.getEstanActual().actual);
			//Parte de juego
			dos.writeInt(gameCanvas.parteJuego);
			insertarRegistro(baos);
			Resultado res;

			//guardamos los resultados preguardados.
			for (int i = 0; i < preguardado.size(); i++) {
				res = (Resultado) preguardado.elementAt(i);
				dos.writeInt(res.id);
				dos.writeUTF(res.info);
				insertarRegistro(baos);
			}
			dos.flush();
			baos.close();
			dos.close();

			//borra el contenido actual de la base de datos buena
			rs = RecordStore.openRecordStore("datosGuardados", true);
			//si la base de datos contiene alguna otra partida guardada,
			// entonces borramos
			//dicha partida guardada. Ms bien sus registros
			if (rs.getNumRecords() != 0) {
				rs.closeRecordStore();
				RecordStore.deleteRecordStore("datosGuardados");
				rs = RecordStore.openRecordStore("datosGuardados", true);
			}

			//sobre escribe, la base de datos temporal, sobre la buen
			byte[] registro;
			for (int i = 1; i <= rsTemp.getNumRecords(); i++) {
				registro = rsTemp.getRecord(i);
				rs.addRecord(registro, 0, registro.length);
			}

			rsTemp.closeRecordStore();
			rs.closeRecordStore();
			RecordStore.deleteRecordStore("temporal");

			Utils.guardandoPartida = false;
		} catch (Exception e) {
			mostrarAlerta("Entre2Almas::guardarDatos::Error desconocido: " + e,
					AlertType.ERROR);
			e.printStackTrace();
		}
	}

	/**
	 * Inserta un registro en el RMS que est actualmente abierto.
	 * 
	 * @param baos
	 *            Stream donde est la informacin a guardar
	 * @throws RecordStoreNotOpenException
	 * @throws InvalidRecordIDException
	 * @throws RecordStoreFullException
	 * @throws RecordStoreException
	 */
	private void insertarRegistro(ByteArrayOutputStream baos)
			throws RecordStoreNotOpenException, InvalidRecordIDException,
			RecordStoreFullException, RecordStoreException {
		byte[] registro = baos.toByteArray();
		rsTemp.addRecord(registro, 0, registro.length);
		baos.reset();
	}

	/**
	 * A la hora de cargar una partida, la cual fue guardada anteriormente, este
	 * mtodo nos devuelve los datos esenciales de la carga. (nFase,mapaActual,
	 * escenario y estancia actual y parteJuego.
	 * 
	 * @return Un array de bytes que contiene todos los datos esenciales
	 */
	public byte[] cargaDatosEsenciales() {
		byte[] registro = null;

		//Borramos todos los datos que estn preguardados.
		preguardado.removeAllElements();

		try {
			rs = RecordStore.openRecordStore("datosGuardados", true);
			//			En el recordStore los parmetros no van de 0 a N
			//sino de 1 a N
			registro = rs.getRecord(1);
			rs.closeRecordStore();
			return registro;
		} catch (Exception e) {
			System.out.println(e);
			try {
				rs.closeRecordStore();
			} catch (RecordStoreNotFoundException e1) {
				mostrarAlerta("Entre2Almas::cargarDatosEsenciales::Error RMS: "
						+ e, AlertType.ERROR);
				e1.printStackTrace();
			} catch (RecordStoreException e1) {
				mostrarAlerta("Entre2Almas::cargarDatosEsenciales::Error RMS: "
						+ e, AlertType.ERROR);
				e1.printStackTrace();
			}
			mostrarAlerta(
					"Entre2Almas::cargarDatosEsenciales::Error desconocido: "
							+ e, AlertType.ERROR);
			return registro;
		}
	}

	/**
	 * A la hora de cargar una partida, la cual fue guardada anteriormente, este
	 * mtodo carga los resultados que hay que ejecutar para que la partida se
	 * quede en el mismo estado que cuando se guardo.
	 */
	public void cargarResultados() {
		try {
			rs = RecordStore.openRecordStore("datosGuardados", true);
			byte[] registro;
			ByteArrayInputStream bais = null;
			DataInputStream dis = null;
			Resultado res;

			//En el recordStore los parmetros no van de 0 a N
			//sino de 1 a N
			for (int i = 2; i <= rs.getNumRecords(); i++) {
				registro = rs.getRecord(i);
				bais = new ByteArrayInputStream(registro);
				dis = new DataInputStream(bais);
				res = new Resultado(dis.readInt(), dis.readUTF());
				gameCanvas.resultados.addElement(res);
				bais.reset();
				dis.reset();
			}

			bais.close();
			dis.close();
			rs.closeRecordStore();
		} catch (Exception e) {
			try {
				rs.closeRecordStore();
			} catch (RecordStoreNotFoundException e1) {
				mostrarAlerta("Entre2Almas::cargarResultados::Error RMS: " + e,
						AlertType.ERROR);
				e1.printStackTrace();
			} catch (RecordStoreException e1) {
				mostrarAlerta("Entre2Almas::cargarResultados::Error RMS: " + e,
						AlertType.ERROR);
				e1.printStackTrace();
			}
			mostrarAlerta("Entre2Almas::cargarResultados::Error desconocido: "
					+ e, AlertType.ERROR);
		}
	}

	/**
	 * Indica, atendiendo al estado actual de la partida, si se puede guardar o
	 * no la partida.
	 * 
	 * @return Si se puede guardar o no la partida.
	 */
	public boolean puedeGuardar() {
		return gameCanvas.puedeGuardar();
	}

	/**
	 * Indica si hay alguna partida guardada.
	 * 
	 * @return Indica si hay alguna partida guardada.
	 */
	public boolean hayGuardada() {
		boolean hay = false;
		try {
			rs = RecordStore.openRecordStore("datosGuardados", true);
			if (rs.getNumRecords() != 0)
				hay = true;
			rs.closeRecordStore();

			return hay;
		} catch (RecordStoreFullException e) {
			mostrarAlerta("Entre2Almas::hayGuardada::Error RMS: " + e,
					AlertType.ERROR);
			e.printStackTrace();
		} catch (RecordStoreNotFoundException e) {
			mostrarAlerta("Entre2Almas::hayGuardada::Error RMS: " + e,
					AlertType.ERROR);
			e.printStackTrace();
		} catch (RecordStoreException e) {
			mostrarAlerta("Entre2Almas::hayGuardada::Error RMS: " + e,
					AlertType.ERROR);
			e.printStackTrace();
		}
		return hay;
	}

	/**
	 * Muestra una alerta por pantalla
	 * 
	 * @param texto
	 *            El texto que contendr la alerta.
	 * @param tipo
	 *            El tipo de alerta a mostrar
	 */
	public void mostrarAlerta(String texto, AlertType tipo) {
		Alert alr = new Alert("Info", texto, null, tipo);
		alr.setTimeout(Alert.FOREVER);
		display.setCurrent(alr);
	}
}